% Randomly generates task sets with specified total U, individual task
% density not exceeding Umax, and the ratio Tmax/Tmin.
% For a fixed mFixed it computes the GMPR and MPR interfaces for each Pi
% in the range [PiMin;PiMax].
% The number of performed simulations is simulationsNumber,
% the generated task set, not having a feasible GMPR or MPR interface, is
% ignored (and is not counted as a valid simulation).
% schedPolicy is a local scheduling policy for the task set, which is GEDF
% or FP. If schedPolicy is not specified, then computations are performed
% for both GEDF and FP.
%
% The simulation results are plotted as U(interface) = f(Pi, schedPolicy).
function simulations_fixed_m_varing_Pi(PiMin, PiMax, U, Umax, Tmaxmin, mFixed, simulationsNumber, schedPolicy)

% If no scheduling policy is specified,
% computations are performed for both GEDF and FP
if nargin < 8
    schedPolicy = '';
end

% Data buffers containing the results of simulations for plotting
GmprUVectors_EDF = {};
MprUVectors_EDF = {};
GmprUVectors_FP = {};
MprUVectors_FP = {};

tasksNumber = 0;

simulationIndex = 1;
while (simulationIndex <= simulationsNumber)
    
    disp('simulation:'); disp(simulationIndex);
    simulationInvalid = 0;
    
    tasks = generate_taskset(U, Umax, Tmaxmin);
    
    % For each Pi in the range [PiMin;PiMax] compute GMPR and MPR
    % interfaces.
    % It considers only task sets, which have feasible GMPR and MPR
    % interfaces for each Pi and a specified m, othervise it is ignored and
    % a new simulation is performed.
    for Pi = PiMin:PiMax
        
        disp(Pi);
        
        Pis(Pi-PiMin+1) = Pi;
        
        mMin = ceil(get_utilization(tasks));
        if (mFixed < mMin)
            disp('No feasible interface for the task set.');
            % Generated taskset cannot be scheduled on less than mMin
            % processors, so there are no feaisble interfaces and
            % another task set is generated
            simulationInvalid = 1;
            break;
        end
        
        
        if (~strcmp(schedPolicy,'FP'))
            if (check_m_feasibility(mFixed, Pi, tasks,'GEDF'))
                gmprInterfacesEDF = compute_gmpr(Pi, mFixed, tasks, 'GEDF');
                mprInterfacesEDF = compute_mpr(Pi, mFixed, tasks, 'GEDF');
            end
        end
        
        if (~strcmp(schedPolicy,'GEDF'))
            if (check_m_feasibility(mFixed, Pi, tasks,'FP'))
                gmprInterfacesFP = compute_gmpr(Pi, mFixed, tasks, 'FP');
                mprInterfacesFP = compute_mpr(Pi, mFixed, tasks, 'FP');
            end
        end
        
        % Check if all interfaces have been generated for a given Pi
        if strcmp(schedPolicy,'GEDF')
            if ((numel(gmprInterfacesEDF) == 0) || (numel(mprInterfacesEDF) == 0))
                simulationInvalid = 1;
                break;
            end
        elseif strcmp(schedPolicy,'FP')
            if ((numel(gmprInterfacesFP) == 0) || (numel(mprInterfacesFP) == 0))
                simulationInvalid = 1;
                break;
            end
        elseif ((strcmp(schedPolicy,'') && ((numel(gmprInterfacesEDF) == 0) || (numel(mprInterfacesEDF) == 0) || (numel(gmprInterfacesFP) == 0) || (numel(mprInterfacesFP) == 0))))
            simulationInvalid = 1;
            break;
        end
        
        
        if (~strcmp(schedPolicy,'FP'))
            % GMPR EDF
            optimalGmprInterfacesEDF = get_optimal_gmpr_interfaces(gmprInterfacesEDF);
            gmprThetaEDF = optimalGmprInterfacesEDF(1).thetas(end);
            gmprU_EDF = gmprThetaEDF / Pi;
            GmprUs_EDF(Pi-PiMin+1) = gmprU_EDF;
            
            % MPR EDF
            mprThetaEDF = mprInterfacesEDF.theta;
            mprU_EDF = mprThetaEDF / Pi;
            MprUs_EDF(Pi-PiMin+1) = mprU_EDF;
        end
        
        if (~strcmp(schedPolicy,'GEDF'))
            % GMPR FP
            optimalGmprInterfacesFP = get_optimal_gmpr_interfaces(gmprInterfacesFP);
            gmprThetaFP = optimalGmprInterfacesFP(1).thetas(end);
            gmprU_FP = gmprThetaFP / Pi;
            GmprUs_FP(Pi-PiMin+1) = gmprU_FP;
            
            % MPR FP
            mprThetaFP = mprInterfacesFP.theta;
            mprU_FP = mprThetaFP / Pi;
            MprUs_FP(Pi-PiMin+1) = mprU_FP;
        end
        
    end % for Pi
    
    if (simulationInvalid)
        % the scenario is invalid
        % as for a generated taskset interfaces do not exist for a given m
        continue;
    end
    
    % Scenario is valid
    tasksNumber = tasksNumber + numel(tasks);
    
    % Put data in buffers
    if (~strcmp(schedPolicy,'FP'))
        GmprUVectors_EDF{simulationIndex} = GmprUs_EDF;
        MprUVectors_EDF{simulationIndex} = MprUs_EDF;
    end
    
    if (~strcmp(schedPolicy,'GEDF'))
        GmprUVectors_FP{simulationIndex} = GmprUs_FP;
        MprUVectors_FP{simulationIndex} = MprUs_FP;
    end
    
    simulationIndex = simulationIndex + 1;
end % while for scenarious

% Average number of tasks over all simulations
avgTasksNumber = tasksNumber / simulationsNumber;


% Analize the results of the experiment
% Compute average values
GmprAverageUs_EDF = [];
MprAverageUs_EDF = [];
GmprAverageUs_FP = [];
MprAverageUs_FP = [];

for Pi = PiMin:PiMax
    GmprUSum_EDF = 0;
    MprUSum_EDF = 0;
    GmprUSum_FP = 0;
    MprUSum_FP = 0;    
    
    for simulation = 1:simulationsNumber
        if (~strcmp(schedPolicy,'FP'))
            % GMPR EDF
            GmprUVector_EDF = GmprUVectors_EDF{simulation};
            GmprUSum_EDF = GmprUSum_EDF + GmprUVector_EDF(Pi-PiMin+1);
            % MPR EDF
            MprUVector_EDF = MprUVectors_EDF{simulation};
            MprUSum_EDF = MprUSum_EDF + MprUVector_EDF(Pi-PiMin+1);
        end
        if (~strcmp(schedPolicy,'GEDF'))
            % GMPR FP
            GmprUVector_FP = GmprUVectors_FP{simulation};
            GmprUSum_FP = GmprUSum_FP + GmprUVector_FP(Pi-PiMin+1);
            % MPR FP
            MprUVector_FP = MprUVectors_FP{simulation};
            MprUSum_FP = MprUSum_FP + MprUVector_FP(Pi-PiMin+1);
        end
    end
    
    if (~strcmp(schedPolicy,'FP'))
        GmprAverageUs_EDF(Pi-PiMin+1) = GmprUSum_EDF/simulationsNumber;
        MprAverageUs_EDF(Pi-PiMin+1) = MprUSum_EDF/simulationsNumber;
    end
    
    if (~strcmp(schedPolicy,'GEDF'))
        GmprAverageUs_FP(Pi-PiMin+1) = GmprUSum_FP/simulationsNumber;
        MprAverageUs_FP(Pi-PiMin+1) = MprUSum_FP/simulationsNumber;
    end
end

% Plot the results
plotTitle = strcat('U=', num2str(U), ', U_{max}=', num2str(Umax), ', Tmax/Tmin=', num2str(Tmaxmin), ', mFixed=', num2str(mFixed), ', navg=', num2str(avgTasksNumber), ', simulationsNumber=', num2str(simulationsNumber));
plot_interface_Us_for_Pis(Pis, GmprAverageUs_EDF, MprAverageUs_EDF, GmprAverageUs_FP, MprAverageUs_FP, plotTitle, schedPolicy);


end